@@ -1,8 +1,7 @@
 /*
  *      filetypesprivate.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -21,7 +20,13 @@
 
 
 #ifndef GEANY_FILETYPES_PRIVATE_H
-#define GEANY_FILETYPES_PRIVATE_H
+#define GEANY_FILETYPES_PRIVATE_H 1
+
+#include "build.h"
+
+#include "gtkcompat.h"
+
+G_BEGIN_DECLS
 
 /* Private GeanyFiletype fields */
 typedef struct GeanyFiletypePrivate
@@ -35,7 +40,21 @@ typedef struct GeanyFiletypePrivate
 	gboolean	xml_indent_tags; /* XML tag autoindentation, for HTML and XML filetypes */
 	GSList		*tag_files;
 	gboolean	warn_color_scheme;
+
+	/* TODO: move to structure in build.h and only put a pointer here */
+	GeanyBuildCommand *filecmds;
+	GeanyBuildCommand *ftdefcmds;
+	GeanyBuildCommand *execcmds;
+	GeanyBuildCommand *homefilecmds;
+	GeanyBuildCommand *homeexeccmds;
+	GeanyBuildCommand *projfilecmds;
+	GeanyBuildCommand *projexeccmds;
+	gint			 project_list_entry;
+	gchar			 *projerror_regex_string;
+	gchar			 *homeerror_regex_string;
 }
 GeanyFiletypePrivate;
 
-#endif
+G_END_DECLS
+
+#endif /* GEANY_FILETYPES_PRIVATE_H */
