@@ -1,8 +1,7 @@
 /*
  *      keybindings.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -111,11 +110,11 @@ static void cb_func_move_tab(guint key_id);
 static void add_popup_menu_accels(void);
 
 
-/** Gets significant modifiers from a GdkModifierType mask. The set of 
- * significant modifiers corresponds to the default modifier mask as returned 
+/** Gets significant modifiers from a GdkModifierType mask. The set of
+ * significant modifiers corresponds to the default modifier mask as returned
  * by @c gtk_accelerator_get_default_mod_mask(). In addition, it improves
  * the Command key handling on OS X by adding @c GEANY_PRIMARY_MOD_MASK
- * when needed. For this reason it is preferred to use this function 
+ * when needed. For this reason it is preferred to use this function
  * instead of @c gtk_accelerator_set_default_mod_mask().
  * @param mods GdkModifierType mask.
  * @return Significant modifiers from the mask.
@@ -2491,7 +2490,7 @@ static gboolean cb_func_format_action(guint key_id)
 			editor_indentation_by_one_space(doc->editor, -1, TRUE);
 			break;
 		case GEANY_KEYS_FORMAT_AUTOINDENT:
-			editor_smart_line_indentation(doc->editor, -1);
+			editor_smart_line_indentation(doc->editor);
 			break;
 		case GEANY_KEYS_FORMAT_TOGGLECASE:
 			on_toggle_case1_activate(NULL, NULL);
