@@ -1,8 +1,7 @@
 /*
  *      stash.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -123,6 +122,7 @@ typedef struct StashPref StashPref;
 
 struct StashGroup
 {
+	guint refcount;				/* ref count for GBoxed implementation */
 	const gchar *name;			/* group name to use in the keyfile */
 	GPtrArray *entries;			/* array of (StashPref*) */
 	gboolean various;		/* mark group for display/edit in stash treeview */
@@ -347,17 +347,27 @@ gint stash_group_save_to_file(StashGroup *group, const gchar *filename,
 }
 
 
+static void free_stash_pref(StashPref *pref)
+{
+	if (pref->widget_type == GTK_TYPE_RADIO_BUTTON)
+		g_free(pref->extra.radio_buttons);
+
+	g_slice_free(StashPref, pref);
+}
+
+
 /** Creates a new group.
  * @param name Name used for @c GKeyFile group.
  * @return Group. */
 GEANY_API_SYMBOL
 StashGroup *stash_group_new(const gchar *name)
 {
-	StashGroup *group = g_new0(StashGroup, 1);
+	StashGroup *group = g_slice_new0(StashGroup);
 
 	group->name = name;
-	group->entries = g_ptr_array_new();
+	group->entries = g_ptr_array_new_with_free_func((GDestroyNotify) free_stash_pref);
 	group->use_defaults = TRUE;
+	group->refcount = 1;
 	return group;
 }
 
@@ -386,27 +396,36 @@ void stash_group_free_settings(StashGroup *group)
 }
 
 
+static StashGroup *stash_group_dup(StashGroup *src)
+{
+	g_atomic_int_inc(&src->refcount);
+
+	return src;
+}
+
+
 /** Frees a group.
  * @param group . */
 GEANY_API_SYMBOL
 void stash_group_free(StashGroup *group)
 {
-	StashPref *entry;
-	guint i;
-
-	foreach_ptr_array(entry, i, group->entries)
+	if (g_atomic_int_dec_and_test(&group->refcount))
 	{
-		if (entry->widget_type == GTK_TYPE_RADIO_BUTTON)
-		{
-			g_free(entry->extra.radio_buttons);
-		}
-		g_slice_free(StashPref, entry);
+		g_ptr_array_free(group->entries, TRUE);
+		g_slice_free(StashGroup, group);
 	}
-	g_ptr_array_free(group->entries, TRUE);
-	g_free(group);
 }
 
 
+/** Gets the GBoxed-derived GType for StashGroup
+ *
+ * @return StashGroup type . */
+GEANY_API_SYMBOL
+GType stash_group_get_type(void);
+
+G_DEFINE_BOXED_TYPE(StashGroup, stash_group, stash_group_dup, stash_group_free);
+
+
 /* Used for selecting groups passed to stash_tree_setup().
  * @c FALSE by default. */
 void stash_group_set_various(StashGroup *group, gboolean various)
@@ -475,7 +494,7 @@ void stash_group_add_integer(StashGroup *group, gint *setting,
  * @param group .
  * @param setting Address of setting variable.
  * @param key_name Name for key in a @c GKeyFile.
- * @param default_value String to copy if the key doesn't exist when loading, or @c NULL. */
+ * @param default_value @nullable String to copy if the key doesn't exist when loading, or @c NULL. */
 GEANY_API_SYMBOL
 void stash_group_add_string(StashGroup *group, gchar **setting,
 		const gchar *key_name, const gchar *default_value)
