@@ -1,6 +1,7 @@
 /*
 *
 *   Copyright (c) 2001-2002, Biswapesh Chattopadhyay
+*   Copyright 2005 The Geany contributors
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -30,7 +31,7 @@
 #include "tm_source_file.h"
 #include "tm_tag.h"
 #include "tm_parser.h"
-#include "tm_ctags_wrappers.h"
+#include "ctags-api.h"
 
 typedef struct
 {
@@ -166,7 +167,7 @@ static char get_tag_access(const char *access)
 }
 
 /*
- Initializes a TMTag structure with information from a tagEntryInfo struct
+ Initializes a TMTag structure with information from a ctagsTag struct
  used by the ctags parsers. Note that the TMTag structure must be malloc()ed
  before calling this function.
  @param tag The TMTag structure to initialize
@@ -174,14 +175,20 @@ static char get_tag_access(const char *access)
  @param tag_entry Tag information gathered by the ctags parser
  @return TRUE on success, FALSE on failure
 */
-static gboolean init_tag(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_entry)
+static gboolean init_tag(TMTag *tag, TMSourceFile *file, const ctagsTag *tag_entry)
 {
 	TMTagType type;
 
 	if (!tag_entry)
 		return FALSE;
 
-	type = tm_parser_get_tag_type(tag_entry->kind->letter, file->lang);
+	type = tm_parser_get_tag_type(tag_entry->kindLetter, tag_entry->lang);
+	if (file->lang != tag_entry->lang)  /* this is a tag from a subparser */
+	{
+		/* check for possible re-definition of subparser type */
+		type = tm_parser_get_subparser_type(file->lang, tag_entry->lang, type);
+	}
+
 	if (!tag_entry->name || type == tm_tag_undef_t)
 		return FALSE;
 
@@ -190,22 +197,24 @@ static gboolean init_tag(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag
 	tag->local = tag_entry->isFileScope;
 	tag->pointerOrder = 0;	/* backward compatibility (use var_type instead) */
 	tag->line = tag_entry->lineNumber;
-	if (NULL != tag_entry->extensionFields.signature)
-		tag->arglist = g_strdup(tag_entry->extensionFields.signature);
-	if ((NULL != tag_entry->extensionFields.scopeName) &&
-		(0 != tag_entry->extensionFields.scopeName[0]))
-		tag->scope = g_strdup(tag_entry->extensionFields.scopeName);
-	if (tag_entry->extensionFields.inheritance != NULL)
-		tag->inheritance = g_strdup(tag_entry->extensionFields.inheritance);
-	if (tag_entry->extensionFields.varType != NULL)
-		tag->var_type = g_strdup(tag_entry->extensionFields.varType);
-	if (tag_entry->extensionFields.access != NULL)
-		tag->access = get_tag_access(tag_entry->extensionFields.access);
-	if (tag_entry->extensionFields.implementation != NULL)
-		tag->impl = get_tag_impl(tag_entry->extensionFields.implementation);
+	if (NULL != tag_entry->signature)
+		tag->arglist = g_strdup(tag_entry->signature);
+	if ((NULL != tag_entry->scopeName) &&
+		(0 != tag_entry->scopeName[0]))
+		tag->scope = g_strdup(tag_entry->scopeName);
+	if (tag_entry->inheritance != NULL)
+		tag->inheritance = g_strdup(tag_entry->inheritance);
+	if (tag_entry->varType != NULL)
+		tag->var_type = g_strdup(tag_entry->varType);
+	if (tag_entry->access != NULL)
+		tag->access = get_tag_access(tag_entry->access);
+	if (tag_entry->implementation != NULL)
+		tag->impl = get_tag_impl(tag_entry->implementation);
 	if ((tm_tag_macro_t == tag->type) && (NULL != tag->arglist))
 		tag->type = tm_tag_macro_with_arg_t;
 	tag->file = file;
+	/* redefine lang also for subparsers because the rest of Geany assumes that
+	 * tags from a single file are from a single language */
 	tag->lang = file->lang;
 	return TRUE;
 }
@@ -419,7 +428,7 @@ static gboolean init_tag_from_file_ctags(TMTag *tag, TMSourceFile *file, FILE *f
 				const gchar *kind = value ? value : key;
 
 				if (kind[0] && kind[1])
-					tag->type = tm_parser_get_tag_type(tm_ctags_get_kind_from_name(kind, lang), lang);
+					tag->type = tm_parser_get_tag_type(ctagsGetKindFromName(kind, lang), lang);
 				else
 					tag->type = tm_parser_get_tag_type(*kind, lang);
 			}
@@ -537,7 +546,7 @@ GPtrArray *tm_source_file_read_tags_file(const gchar *tags_file, TMParserType mo
 		return NULL; /* early out on error */
 	}
 	else
-	{	/* We read the first line for the format specification. */
+	{	/* We read (and discard) the first line for the format specification. */
 		if (buf[0] == '#' && strstr((gchar*) buf, "format=pipe") != NULL)
 			format = TM_FILE_FORMAT_PIPE;
 		else if (buf[0] == '#' && strstr((gchar*) buf, "format=tagmanager") != NULL)
@@ -562,8 +571,9 @@ GPtrArray *tm_source_file_read_tags_file(const gchar *tags_file, TMParserType mo
 				format = TM_FILE_FORMAT_PIPE;
 			else if (tab_cnt > 1)
 				format = TM_FILE_FORMAT_CTAGS;
+			/* reset the file pointer, to start reading again from the beginning */
+			rewind(fp);
 		}
-		rewind(fp); /* reset the file pointer, to start reading again from the beginning */
 	}
 
 	file_tags = g_ptr_array_new();
@@ -633,7 +643,7 @@ static void update_python_arglist(const TMTag *tag, TMSourceFile *current_source
 }
 
 /* new parsing pass ctags callback function */
-static gboolean ctags_pass_start(void *user_data)
+static bool ctags_pass_start(void *user_data)
 {
 	TMSourceFile *current_source_file = user_data;
 
@@ -642,7 +652,7 @@ static gboolean ctags_pass_start(void *user_data)
 }
 
 /* new tag ctags callback function */
-static gboolean ctags_new_tag(const tagEntryInfo *const tag,
+static bool ctags_new_tag(const ctagsTag *const tag,
 	void *user_data)
 {
 	TMSourceFile *current_source_file = user_data;
@@ -663,7 +673,7 @@ static gboolean ctags_new_tag(const tagEntryInfo *const tag,
 }
 
 /* Initializes a TMSourceFile structure from a file name. */
-static gboolean tm_source_file_init(TMSourceFile *source_file, const char *file_name, 
+static gboolean tm_source_file_init(TMSourceFile *source_file, const char *file_name,
 	const char* name)
 {
 	GStatBuf s;
@@ -699,7 +709,7 @@ static gboolean tm_source_file_init(TMSourceFile *source_file, const char *file_
 	if (name == NULL)
 		source_file->lang = TM_PARSER_NONE;
 	else
-		source_file->lang = tm_ctags_get_named_lang(name);
+		source_file->lang = ctagsGetNamedLang(name);
 
 	return TRUE;
 }
@@ -792,57 +802,33 @@ gboolean tm_source_file_parse(TMSourceFile *source_file, guchar* text_buf, gsize
 {
 	const char *file_name;
 	gboolean retry = TRUE;
-	gboolean parse_file = FALSE;
-	gboolean free_buf = FALSE;
 
 	if ((NULL == source_file) || (NULL == source_file->file_name))
 	{
 		g_warning("Attempt to parse NULL file");
 		return FALSE;
 	}
-	
+
 	if (source_file->lang == TM_PARSER_NONE)
 	{
 		tm_tags_array_free(source_file->tags_array, FALSE);
 		return FALSE;
 	}
-	
-	file_name = source_file->file_name;
-	
-	if (!use_buffer)
-	{
-		GStatBuf s;
-		
-		/* load file to memory and parse it from memory unless the file is too big */
-		if (g_stat(file_name, &s) != 0 || s.st_size > 10*1024*1024)
-			parse_file = TRUE;
-		else
-		{
-			if (!g_file_get_contents(file_name, (gchar**)&text_buf, (gsize*)&buf_size, NULL))
-			{
-				g_warning("Unable to open %s", file_name);
-				return FALSE;
-			}
-			free_buf = TRUE;
-		}
-	}
 
-	if (!parse_file && (NULL == text_buf || 0 == buf_size))
+	file_name = source_file->file_name;
+
+	if (use_buffer && (NULL == text_buf || 0 == buf_size))
 	{
 		/* Empty buffer, "parse" by setting empty tag array */
 		tm_tags_array_free(source_file->tags_array, FALSE);
-		if (free_buf)
-			g_free(text_buf);
 		return TRUE;
 	}
 
 	tm_tags_array_free(source_file->tags_array, FALSE);
 
-	tm_ctags_parse(parse_file ? NULL : text_buf, buf_size, file_name,
+	ctagsParse(use_buffer ? text_buf : NULL, buf_size, file_name,
 		source_file->lang, ctags_new_tag, ctags_pass_start, source_file);
 
-	if (free_buf)
-		g_free(text_buf);
 	return !retry;
 }
 
@@ -852,7 +838,7 @@ gboolean tm_source_file_parse(TMSourceFile *source_file, guchar* text_buf, gsize
 */
 const gchar *tm_source_file_get_lang_name(TMParserType lang)
 {
-	return tm_ctags_get_lang_name(lang);
+	return ctagsGetLangName(lang);
 }
 
 /* Gets the language index for \a name.
@@ -861,5 +847,5 @@ const gchar *tm_source_file_get_lang_name(TMParserType lang)
 */
 TMParserType tm_source_file_get_named_lang(const gchar *name)
 {
-	return tm_ctags_get_named_lang(name);
+	return ctagsGetNamedLang(name);
 }
